@@ -1,5 +1,5 @@
 /*  QuickSynergy -- a GUI for synergy
- *  Copyright (C) 2006, 2007, 2008 Cesar L. B. Silveira, Otavio C. Cordeiro
+ *  Copyright (C) 2006, 2007, 2008, 2009 Cesar L. B. Silveira, Otavio C. Cordeiro
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,15 +22,18 @@
 #include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
+#include <sys/stat.h>
 #include <sys/wait.h>
 #include <signal.h>
 #include <unistd.h>
+#include <glib.h>
 #include <gtk/gtk.h>
 #include "callbacks.h"
 #include "synergy_config.h"
+#include "ui.h"
 #include "intl.h"
 
-#ifdef HAVE_GTK_2_10
+#if GTK_CHECK_VERSION(2,10,0)
 static GtkUIManager *ui_manager = NULL;
 static GtkActionGroup *action_group;
 
@@ -49,15 +52,9 @@ static GtkActionEntry popup_menu_actions[] = {
 };
 #endif
 
-pid_t pid;
-
 extern GtkWidget *main_window;
 extern GtkWidget *notebook;
-extern GtkWidget *hostname_entry;
-extern GtkWidget *table;
-extern GtkWidget *vbox0;
-extern int synergy_running;
-extern const guint8 qslogo[];
+extern GtkWidget *start_button;
 
 gboolean entry_focus_in_event(
         GtkWidget *widget, GdkEventFocus *event, gpointer data) {
@@ -66,7 +63,7 @@ gboolean entry_focus_in_event(
     text = (char *) gtk_entry_get_text(GTK_ENTRY(widget));
     
     /* if the text on the entry is the default text (Above, Below, 
-     * Left, Right), erae it when the entry gets focus */
+     * Left, Right), erase it when the entry gets focus */
     if(!strcmp(text, (const char *) data))
         gtk_entry_set_text(GTK_ENTRY(widget), "");
 
@@ -87,6 +84,40 @@ gboolean entry_focus_out_event(
     return FALSE;
 }
 
+void entry_changed_cb(GtkEntry *entry, gpointer data) {
+    *((char **) data) = (char *) gtk_entry_get_text(entry);
+}
+
+void notebook_page_switched(GtkNotebook *notebook, GtkNotebookPage *page,
+        guint page_num, gpointer user_data) {
+    if (page_num == 2) {
+        gtk_widget_set_sensitive(start_button, FALSE);
+    } else {
+        gtk_widget_set_sensitive(start_button, TRUE);
+    }
+}
+
+void browse_button_clicked(GtkButton *button, gpointer userdata) {
+    GtkWidget *synergy_path_entry = (GtkWidget *) userdata;
+    GtkWidget *dialog;
+    
+    dialog = gtk_file_chooser_dialog_new(_("Choose path to synergy binaries"),
+        GTK_WINDOW(main_window), GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
+        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+        GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+        NULL);
+        
+    if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
+        gchar *folder;
+        
+        folder = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
+        gtk_entry_set_text(synergy_path_entry, folder);
+        g_free(folder);
+    }
+    
+    gtk_widget_destroy(dialog);
+}
+
 void about_button_clicked(GtkWidget *widget, gpointer data) {
     GtkWidget *window = (GtkWidget *) data;
     
@@ -97,6 +128,8 @@ void about_button_clicked(GtkWidget *widget, gpointer data) {
         "Thomas Langewouters (patches and suggestions)",
         "Magnus Boman (patches)",
         "Todd Wease (patches)",
+        "Avadh Patel (screen name patch)",
+        "Stephan Kleine (openSUSE packaging and patches)",
         NULL
     };
     
@@ -107,9 +140,9 @@ void about_button_clicked(GtkWidget *widget, gpointer data) {
     };
     
     static const gchar *copyright =
-        "Copyright \xc2\xa9 2006-2008 César L. B. Silveira, Otávio C. Cordeiro";
+        "Copyright \xc2\xa9 2006-2009 César L. B. Silveira, Otávio C. Cordeiro";
     
-    GdkPixbuf *logo = gdk_pixbuf_new_from_inline(-1, qslogo, FALSE, NULL);
+    GdkPixbuf *logo = make_logo();
 
     gtk_show_about_dialog(GTK_WINDOW(window),
                     "comments", _("A graphical user interface for synergy"),
@@ -129,65 +162,113 @@ void about_button_clicked(GtkWidget *widget, gpointer data) {
 }
 
 void start_button_clicked(GtkWidget *widget, gpointer data) {
-    const char *env_home = getenv("HOME");
-    char *filename, *hostname;
+    qs_state_t *state = (qs_state_t *) data;
+    const char *home_dir = getenv("HOME");
+    struct stat finfo;
+    gchar *cmd;
     int status;
     
-    if(!synergy_running) {
+    chdir(home_dir);
+    
+    if(!state->running) {
         if(gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook)) == 0) {    
-            save_config();
-            save_synergy_config();
-        
-            filename = (char *) malloc(
-                (strlen(env_home) + strlen("/.quicksynergy/synergy.conf")) * 
-                sizeof(char));
-        
-            sprintf(filename, "%s%s", env_home, "/.quicksynergy/synergy.conf");
+            save_config(state);
+            save_synergy_config(state);
             
-            if((pid = fork()) == 0) {
-                execlp("synergys", "synergys", "-f", "--config",  filename, NULL);
+            cmd = g_strjoin("/", state->synergy_path, "synergys", NULL);
+            
+            if (stat(cmd, &finfo) == -1) {
+                GtkWidget *dialog;
+                
+                dialog = gtk_message_dialog_new(GTK_WINDOW(main_window),
+                    GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
+                    _("%s: File not found."), cmd);
+                gtk_window_set_title(GTK_WINDOW(dialog), _("Error"));
+                
+                gtk_dialog_run(GTK_DIALOG(dialog));
+                
+                gtk_widget_destroy(dialog);
+                
+                g_free(cmd);
+                
+                return;
             }
             
+            state->pid = fork();
+        
+            if(state->pid == 0) {
+                execlp(cmd, cmd, "-f", "--config",
+                    ".quicksynergy/synergy.conf", NULL);
+            }
+            
+            g_free(cmd);
+            
             gtk_widget_set_sensitive(notebook, FALSE);
             
-            free(filename);
-            
-            synergy_running = SYNERGY_SERVER_RUNNING;
+            state->running = SYNERGY_SERVER_RUNNING;
         }
         else if(gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook)) == 1) {
-            save_config();
-
-            hostname = 
-                (char *) gtk_entry_get_text(GTK_ENTRY(hostname_entry));
-            if((pid = fork()) == 0) {
-                execlp("synergyc", "synergyc", "-f", hostname, NULL);
+            save_config(state);
+            
+            cmd = g_strjoin("/", state->synergy_path, "synergyc", NULL);
+            
+            if (stat(cmd, &finfo) == -1) {
+                GtkWidget *dialog;
+                
+                dialog = gtk_message_dialog_new(GTK_WINDOW(main_window),
+                    GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
+                    _("%s: File not found."), cmd);
+                gtk_window_set_title(GTK_WINDOW(dialog), _("Error"));
+                
+                gtk_dialog_run(GTK_DIALOG(dialog));
+                
+                gtk_widget_destroy(dialog);
+                
+                g_free(cmd);
+                
+                return;
             }
 
-            synergy_running = SYNERGY_CLIENT_RUNNING;
+            state->pid = fork();
+
+            if(state->pid == 0) {
+                if(!strcmp(state->client_name, "")) {
+                    execlp(cmd, cmd, "-f", state->hostname, NULL);
+                } else {
+                    execlp(cmd, cmd, "-f", "--name", state->client_name,
+                        state->hostname, NULL);
+                }
+            }
+            
+            g_free(cmd);
+
+            state->running = SYNERGY_CLIENT_RUNNING;
         }
         
         gtk_button_set_label(GTK_BUTTON(widget), GTK_STOCK_MEDIA_STOP);
         gtk_widget_set_sensitive(notebook, FALSE);
     }
     else {
-        kill(pid, SIGTERM);
+        kill(state->pid, SIGTERM);
 
         wait(&status);
         
         gtk_button_set_label(GTK_BUTTON(widget), GTK_STOCK_EXECUTE);
         gtk_widget_set_sensitive(notebook, TRUE);
         
-        synergy_running = 0;
+        state->running = 0;
     }
 }
 
-#ifdef HAVE_GTK_2_10
+#if GTK_CHECK_VERSION(2,10,0)
 void quicksynergy_quit(GtkWidget *widget, gpointer data) {
-    if(synergy_running) {
-        kill(pid, SIGTERM);
+    qs_state_t *state = (qs_state_t *) data;
+    
+    if(state->running) {
+        kill(state->pid, SIGTERM);
     }
     
-    save_config();
+    save_config(state);
     
     gtk_main_quit();
 }
@@ -198,36 +279,42 @@ void close_button_clicked(GtkWidget *widget, gpointer data) {
 
 gboolean delete_event(GtkWidget *widget, GdkEvent *event, gpointer data) {
     gtk_widget_hide_all(widget);
+    
+    return TRUE;
 }
 #else
 void close_button_clicked(GtkWidget *widget, gpointer data) {   
-    if(synergy_running) {
-        kill(pid, SIGTERM);
+    qs_state_t *state = (qs_state_t *) data;
+    
+    if(state->running) {
+        kill(state->pid, SIGTERM);
     }
     
-    save_config();
+    save_config(state);
     
     gtk_main_quit();
 }
 
 gboolean delete_event(GtkWidget *widget, GdkEvent *event, gpointer data) {
-    int status;
+    qs_state_t *state = (qs_state_t *) data;
     
-    if(synergy_running) {
-        kill(pid, SIGTERM);
+    if(state->running) {
+        kill(state->pid, SIGTERM);
     }
     
-    save_config();
+    save_config(state);
     
     return FALSE;
 }
 #endif
 
-#ifdef HAVE_GTK_2_10
+#if GTK_CHECK_VERSION(2,10,0)
 void show_main_window(GtkWidget *widget, gpointer data) {
+    qs_state_t *state = (qs_state_t *) data;
+    
     gtk_widget_show_all(main_window);
     
-    if(!synergy_running || synergy_running == SYNERGY_SERVER_RUNNING) {
+    if(!state->running || state->running == SYNERGY_SERVER_RUNNING) {
         gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), 0);
     }
     else {
@@ -235,7 +322,8 @@ void show_main_window(GtkWidget *widget, gpointer data) {
     }
 }
 
-void status_icon_popup(GtkStatusIcon *status_icon, guint button, guint activate_time, gpointer data) {
+void status_icon_popup(GtkStatusIcon *status_icon, guint button,
+        guint activate_time, gpointer data) {
     GtkWidget *popup_menu;
     
     if(!ui_manager) {
@@ -245,7 +333,7 @@ void status_icon_popup(GtkStatusIcon *status_icon, guint button, guint activate_
 #ifdef ENABLE_NLS
         gtk_action_group_set_translation_domain(action_group, PACKAGE);
 #endif
-        gtk_action_group_add_actions(action_group, popup_menu_actions, 2, NULL);
+        gtk_action_group_add_actions(action_group, popup_menu_actions, 2, data);
         
         gtk_ui_manager_insert_action_group(ui_manager, action_group, 0);
         gtk_ui_manager_add_ui_from_string(ui_manager, popup_menu_actions_xml,
@@ -260,3 +348,4 @@ void status_icon_popup(GtkStatusIcon *status_icon, guint button, guint activate_
         gtk_status_icon_position_menu, status_icon, button, activate_time);
 }
 #endif
+
