@@ -1,5 +1,5 @@
 /*  QuickSynergy -- a GUI for synergy
- *  Copyright (C) 2006, 2007, 2008 Cesar L. B. Silveira, Otavio C. Cordeiro
+ *  Copyright (C) 2006, 2007, 2008, 2009 Cesar L. B. Silveira, Otavio C. Cordeiro
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -29,11 +29,14 @@ enum synergy_mode {
 
 gboolean entry_focus_in_event(GtkWidget *widget, GdkEventFocus *event, gpointer data);
 gboolean entry_focus_out_event(GtkWidget *widget, GdkEventFocus *event, gpointer data);
+void entry_changed_cb(GtkEntry *entry, gpointer data);
+void notebook_page_switched(GtkNotebook *notebook, GtkNotebookPage *page, guint page_num, gpointer user_data);
 void about_button_clicked(GtkWidget *widget, gpointer data);
 void start_button_clicked(GtkWidget *widget, gpointer data);
 void close_button_clicked(GtkWidget *widget, gpointer data);
+void browse_button_clicked(GtkButton *button, gpointer userdata);
 gboolean delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data);
-#ifdef HAVE_GTK_2_10
+#if GTK_CHECK_VERSION(2,10,0)
 void quicksynergy_quit(GtkWidget *widget, gpointer data);
 void show_main_window(GtkWidget *widget, gpointer data);
 void status_icon_popup(GtkStatusIcon *status_icon, guint button, guint activate_time, gpointer user_data);
