@@ -1,5 +1,5 @@
 /*  QuickSynergy -- a GUI for synergy
- *  Copyright (C) 2006, 2007, 2008 Cesar L. B. Silveira, Otavio C. Cordeiro
+ *  Copyright (C) 2006, 2007, 2008, 2009 Cesar L. B. Silveira, Otavio C. Cordeiro
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,10 +19,14 @@
 #ifndef SYNERGY_CONFIG_H
 #define SYNERGY_CONFIG_H
 
-#include <gtk/gtk.h>
+#include "ui.h"
 
-void load_config();
-void save_config();
-void save_synergy_config();
+#define QS_CONF_DIR             ".quicksynergy/"
+#define QS_CONF_FILE            "quicksynergy.conf"
+#define QS_SYNERGY_CONF_FILE    "synergy.conf"
+
+qs_state_t *load_config(void);
+void save_config(qs_state_t *state);
+void save_synergy_config(qs_state_t *state);
 
 #endif /* SYNERGY_CONFIG_H */
